:: standard P2J server startup script
@echo off
setlocal enableextensions enabledelayedexpansion

:: defaults
set cpath=.;..\lib\hotel.jar;..\lib\p2j.jar;
set debug=0
set suspend=n
set test=n
set hprof=
set prog=java
set srvr=-server
set heap=768
set outlog=stdout.log
set errlog=server.log
set cfg=server.xml
set port=
set portbase=3333
set portstep=100
set mode=
set statcode=n
set instance=0
set batch=

set usage[0]=Usage: %0 [-dyptksw1] [-bprocess] [-z cp] [-h heap] [-l log] [-c cfg] [-i inst]
set usage[1]=Where:
set usage[2]=-d       = enable JVM debug mode (debugger port will be set as 2080 + instance #)
set usage[3]=-y       = in JVM debug mode enable, suspend the JVM on startup (does not suspend by default)
set usage[4]=-p       = enable JVM hprof (profiling output to gc.log)
set usage[5]=-t       = test mode (displays the command but doesn't execute)
set usage[6]=-k       = kill mode (shuts down the specified/default instance)
set usage[7]=-s       = status mode (displays the status of the specified/default instance)
set usage[8]=-w       = wait mode (does not return until the specified/default instance is STATUS_RUNNING)
set usage[9]=-1       = enable C1 HotSpot compiler (client compiler for the JVM)
set usage[10]=-z      = override the classpath which would otherwise be set to %cpath%
set usage[11]=-h      = set JVM max heap size (must be an integer number of MB)
set usage[12]=-l      = log file name (all STDERR is redirected to this file, defaults to ./server.log)
set usage[13]=-c      = config file name (P2J bootstrap config file name, defaults to server.xml)
set usage[14]=-i      = server instance number (0..9, assigns the P2J server port, defaults to 0)
set usage[15]=-b      = launch a program associated with a process account

set argCount=0
for %%x in (%*) do (
   set /A argCount=!argCount!+1
   set "argVec[!argCount!]=%%~x"
)
set srvr=-server
set /a i=1
:while1
   set carg=!argVec[%i%]!
   
   if "%carg%" equ "-d" (
      set debug=1
   ) else if "%carg%" equ "-y" (
      set suspend=y
   ) else if "%carg%" equ "-p" (
      set hprof=-Xrunhprof:heap=dump,doe=y,format=b,file=gc.log
   ) else if "%carg%" equ "-t" (
      set prog=echo %prog% 
      set test=y
   ) else if "%carg%" equ "-k" (
      set mode=-k
      set statcode=y 
   ) else if "%carg%" equ "-s" (
      set mode=-s
      set statcode=y
   ) else if "%carg%" equ "-w" (
      set mode=-w -1
      set statcode=y 
   ) else if "%carg%" equ "-b" (
      set /A "i=i+1"
      call :setvar batch !i!
      set batch=-b !batch!
   ) else if "%carg%" equ "-1" (
      set srvr=
   ) else if "%carg%" equ "-z" (
      set /A "i=i+1"
      call :setvar cpath !i!
   ) else if "%carg%" equ "-h" (
      set /A "i=i+1"
      call :setvar heap !i!
   ) else if "%carg%" equ "-l" (
      set /A "i=i+1"
      call :setvar log !i!
   ) else if "%carg%" equ "-c" (
      set /A "i=i+1"
      call :setvar cfg !i!
   ) else if "%carg%" equ "-i" (
      set /A "i=i+1"
      call :setvar instance !i!
   ) else if "%carg%" equ "help" (
      for /L %%a in (0,1,14) do call echo.%%usage[%%a]%%
   )
   set /a i=%i%+1
   if %i% leq %argCount% ( goto :while1 )

:: final arg prep
set cpath=-classpath %cpath%
set /A dport=(2080 + %instance%)
set /A sport=(%portbase% + %instance%)
set /A iport=(%portbase% + %instance% + %portstep%)
set iport=net:server:insecure_port=%iport%
set port=net:connection:secure=true net:server:secure_port=%sport% %iport%

if %debug% equ 1 ( set dtxt=-Xdebug -Xnoagent -Djava.compiler=NONE -Xrunjdwp:transport=dt_socket,address=%dport%,server=y,suspend=%suspend% ) else ( set dtxt= )

set maxheap=-Xmx%heap%m

:: prepare command line to run the server
set serverCmd=%prog% %maxheap% %srvr% %dtxt% %cpath% -Djava.system.class.loader=com.goldencode.p2j.classloader.MultiClassLoader com.goldencode.p2j.main.ServerDriver %mode% %batch% %cfg% %port%
:: long lines protection 
if "%test%" equ "y" (
   set serverCmd="%serverCmd%
) else (
   set serverCmd="%serverCmd% 1>%outlog% 2>%errlog%"
)

:: clean up environment variables no longer needed, this is mandatory for web client to operate
:: normally, otherwise some unexpected errors will occur like The parameter is incorrect with
:: CreateProcessWithLogonW.
set usage[0]=
set usage[1]=
set usage[2]=
set usage[3]=
set usage[4]=
set usage[5]=
set usage[6]=
set usage[7]=
set usage[8]=
set usage[9]=
set usage[10]=
set usage[11]=
set usage[12]=
set usage[13]=
set usage[14]=
set dport=
set sport=
set iport=
set portbase=
set portstep=
set instance=
set i=
set argCount=
set carg=
set debug=
set dport=
set heap=
set suspend=
set prog=
set maxheap=
set srvr=
set dtxt=
set cpath=
set mode=
set cfg=
set port=
set outlog=
set errlog=

:: run the P2J server
CMD.EXE /C %serverCmd%

set rc=%errorlevel%

if "%statcode%" equ "y" (
   switch %rc%
   case 0
      set status=STATUS_RUNNING
   case 1
      set status=STATUS_TIMEOUT
   case 2
      set status=STATUS_UNKNOWN
   case 3
      set status=STATUS_STOPPED
   endswitch
   
   echo Result = %status%
fi

endlocal

goto:eof

:setvar
set k=%2
set %1=!argVec[%k%]!
goto:eof

:getPID  [RtnVar]
::
:: Store the Process ID (PID) of the currently running script in environment variable RtnVar.
:: If called without any argument, then simply write the PID to stdout.
::
setlocal disableDelayedExpansion
:getLock
set "lock=%temp%\%~nx0.%time::=.%.lock"
set "uid=%lock:\=:b%"
set "uid=%uid:,=:c%"
set "uid=%uid:'=:q%"
set "uid=%uid:_=:u%"
setlocal enableDelayedExpansion
set "uid=!uid:%%=:p!"
endlocal & set "uid=%uid%"
2>nul ( 9>"%lock%" (
  for /f "skip=1" %%A in (
    'wmic process where "name='cmd.exe' and CommandLine like '%%<%uid%>%%'" get ParentProcessID'
  ) do for %%B in (%%A) do set "PID=%%B"
  (call )
))||goto :getLock
del "%lock%" 2>nul
endlocal & if "%~1" equ "" (echo(%PID%) else set "%~1=%PID%"
exit /b
