#!/bin/bash

VERBOSE=1
ANALYTICS=0
while getopts "na" opt; do
   case $opt in
      n  ) VERBOSE=0 ;;
      a  ) ANALYTICS=1 ;;
   esac
done
shift $(($OPTIND - 1))

function verbose () 
{
    [[ $VERBOSE -eq 1 ]] && return 0 || return 1
}

sed_prepare_res=""
function sed_prepare
{
   res="$1"
   res=$(echo $res | sed -e 's/\\/\\\\/g') # \
   res=$(echo $res | sed -e 's/\//\\\//g') # /
   res=$(echo $res | sed -e 's/\[/\\\[/g') # [
   res=$(echo $res | sed -e 's/\]/\\\]/g') # ]
   res=$(echo $res | sed -e 's/\-/\\\-/g') # -
   res=$(echo $res | sed -e 's/\&/\\\&/g') # &
   res=$(echo $res | sed -e 's/\:/\\\:/g') # :
   res=$(echo $res | sed -e 's/\*/\\\*/g') # *
   res=$(echo $res | sed -e 's/\./\\\./g') # .
   res=$(echo $res | sed -e 's/\$/\\\$/g') # $
   res=$(echo $res | sed -e 's/\"/\\\"/g') # "
   # res=$(echo $res | sed -e 's/\{/\\\{/g') # {
   # res=$(echo $res | sed -e 's/\}/\\\}/g') # }
   
   sed_prepare_res="$res"
}

appname=""

# 1. conversion inputs
# [include-spec] *.[fhi] - default
include_spec="*.[fhi]"
# [program-spec] *.[pPwW] - default
program_spec="*.[pPwW]"
# [pkgroot] com.goldencode.hotel default: test.[appname]
pkgroot=""
# [pkgrootfolder] com/goldencode/hotel
#   automated from [pkgroot] - always use "/"
pkgrootfolder=""
# [propath]    
#   p2j.cfg.linux:   "${P2J_HOME}:${P2J_HOME}/abl"
#   p2j.cfg.windows: "${P2J_HOME};${P2J_HOME}\abl"
#   directory.xml: ".: " -> with ": " separator, from this it computes the linux/windows versions
propath=".:"
propathsep=":"
propathcfg=""

# 2. database inputs:
# [dbnames] hotel1,hotel2
dbnames=""
# [dbadmin] fwd_admin
dbadmin=""
# [dbadminpass] admin
dbadminpass=""
# [dbtype] h2
dbtype="h2"
# [dbhost] localhost
dbhost="localhost"
# [dbport] 5433
dbport="5433"
# [dbuser] fwd_user
dbuser=""
# [dbuserpass] user
dbuserpass=""
#[dmoindex] com/goldencode/hotel/dmo/dmo_index.xml [pkgrootfolder]/dmo/dmo_index.xml
#   automated...
dmoindex=""

# 3. runtime setup 
# [spawner-path] /opt/spawner/spawn default: $PWD/deploy/spawner/spawn
# automated
spawner_path=""
# [client-start-dir] /home/ca/workspace/hotel/deploy/client default: $PWD/deploy/client
# automated
client_start_dir=""
# [p2j-entry] com.goldencode.hotel.Login
p2j_entry=""
# [dateFormat] mdy
dateFormat="mdy"
# [numberGroupSep] ,
numberGroupSep=","
# [numberDecimalSep] .
numberDecimalSep="."
# [path-separator] :
path_separator=":"
# [file-separator] /
file_separator="/"
# [case-sensitive] TRUE
case_sensitive="TRUE"

read -e -p "Enter your application name (only letters or digits, first character a letter): " appname
pkgroot="test.$appname"

# 1. conversion inputs
verbose && echo "1. Enter conversion configurations for your project"
read -e -p "Edit the regex which matches all your include files (enter for default): " -i "$include_spec" include_spec
read -e -p "Edit the regex which matches all your program files (enter for default): " -i "$program_spec" program_spec
if [ "$ANALYTICS" == "0" ] ;
then
   read -e -p "Edit the root package name - Java format (enter for default): " -i "$pkgroot" pkgroot
fi
# pkgrootfolder - automated
read -e -p "Enter the PROPATH from your progress.ini file (enter for default): " -i "$propath" propath

# 2. database inputs
verbose && echo "2. Enter database configuration for your project"
read -e -p "Enter the legacy database name(s), separated by a comma: " -i "$dbnames" dbnames
if [ "$ANALYTICS" == "0" ] ;
then
   read -e -p "Enter the database type (h2 or postgresql): " -i "$dbtype" dbtype
   if [ "$dbtype" == "postgresql" ] ;
   then
      read -e -p "Edit the PostgreSQL host name for your imported database: " -i "$dbhost" dbhost
      read -e -p "Edit the PostgreSQL port for your imported database: " -i "$dbport" dbport
   fi
   read -e -p "Enter the SQL admin for your imported database: " -i "$dbadmin" dbadmin
   read -e -p "Enter the SQL admin password for your imported database: " -i "$dbadminpass" dbadminpass
   read -e -p "Enter the SQL user for your imported database: " -i "$dbuser" dbuser
   read -e -p "Enter the SQL user password for your imported database: " -i "$dbuserpass" dbuserpass
   # dmoindex - automated
   
   # 3. runtime setup
   verbose && echo "3. Enter runtime configuration for your project"
   read -e -p "Enter the date format from your progress.ini (enter for default): " -i "$dateFormat" dateFormat
   read -e -p "Enter the number group separator from your progress.ini (enter for default): " -i "$numberGroupSep" numberGroupSep
   read -e -p "Enter the number decimal separator from your progress.ini (enter for default): " -i "$numberDecimalSep" numberDecimalSep
   read -e -p "Enter the full class name associated with the entry point for your application: " -i "$pkgroot.$p2j_entry" p2j_entry
fi
  
# build automated values
pkgrootfolder=`echo $pkgroot |sed  's/\./\//g'`
dmoindex="$pkgrootfolder/dmo/dmo_index.xml"
spawner_path="/opt/spawner/spawn"
client_start_dir="$PWD/deploy/client"

path_separator=";"
file_separator="\\"
case_sensitive="FALSE"
unix_escapes="FALSE"
propathsep=";"

propathentries=$(echo $propath | tr "$propathsep" "\n")
propathcfg="\${P2J_HOME}:\${P2J_HOME}/abl"
searchpath="."
for pentry in $propathentries
do
   if [ "$pentry" != "." ]
   then
      ppentry=`echo $pentry | sed -e 's/\\\\/\\//g'`
      propathcfg="$propathcfg:\${P2J_HOME}/abl/$ppentry"
      
      searchpath="$searchpath:$ppentry"
   fi
done
user="$USER"

export IFS=","
for dbname in $dbnames; do
   break
done
export IFS=

sed_prepare "$appname"
appname="$sed_prepare_res"
sed_prepare "$include_spec"
include_spec="$sed_prepare_res"
sed_prepare "$program_spec"
program_spec="$sed_prepare_res"
sed_prepare "$pkgroot"
pkgroot="$sed_prepare_res"
sed_prepare "$pkgrootfolder"
pkgrootfolder="$sed_prepare_res"
sed_prepare "$propath"
propath="$sed_prepare_res"
sed_prepare "$propathcfg"
propathcfg="$sed_prepare_res"
sed_prepare "$dbnames"
dbnames="$sed_prepare_res"
sed_prepare "$dbhost"
dbhost="$sed_prepare_res"
sed_prepare "$dbport"
dbport="$sed_prepare_res"
sed_prepare "$dbuser"
dbuser="$sed_prepare_res"
sed_prepare "$dbuserpass"
dbuserpass="$sed_prepare_res"
sed_prepare "$dbadmin"
dbadmin="$sed_prepare_res"
sed_prepare "$dbadminpass"
dbadminpass="$sed_prepare_res"
sed_prepare "$dmoindex"
dmoindex="$sed_prepare_res"
sed_prepare "$dateFormat"
dateFormat="$sed_prepare_res"
sed_prepare "$numberGroupSep"
numberGroupSep="$sed_prepare_res"
sed_prepare "$numberDecimalSep"
numberDecimalSep="$sed_prepare_res"
sed_prepare "$p2j_entry"
p2j_entry="$sed_prepare_res"
sed_prepare "$spawner_path"
spawner_path="$sed_prepare_res"
sed_prepare "$client_start_dir"
client_start_dir="$sed_prepare_res"
sed_prepare "$path_separator"
path_separator="$sed_prepare_res"
sed_prepare "$file_separator"
file_separator="$sed_prepare_res"
sed_prepare "$case_sensitive"
case_sensitive="$sed_prepare_res"
sed_prepare "$unix_escapes"
unix_escapes="$sed_prepare_res"
sed_prepare "$user"
user="$sed_prepare_res"
sed_prepare "$searchpath"
searchpath="$sed_prepare_res"

dbh2="false"
dbpostgresql="false"

if [ "$dbtype" == "h2" ] ;
then
   dbh2="true"
fi
if [ "$dbtype" == "postgresql" ] ;
then
   dbpostgresql="true"
fi

sed -e "s/\[appname\]/$appname/g" \
    -e "s/\[program\-spec\]/$program_spec/g" \
    -e "s/\[dbh2\]/$dbh2/g" \
    -e "s/\[dbpostgresql\]/$dbpostgresql/g" \
    -e "s/\[dbhost\]/$dbhost/g" \
    -e "s/\[dbport\]/$dbport/g" \
    -e "s/\[dbnames\]/$dbnames/g" \
    -e "s/\[dbuser\]/$dbuser/g" \
    -e "s/\[dbuserpass\]/$dbuserpass/g" \
    -e "s/\[dbadmin\]/$dbadmin/g" \
    -e "s/\[dbadminpass\]/$dbadminpass/g" \
    -e "s/\[pkgrootfolder\]/$pkgrootfolder/g" \
    build.properties.template > build.properties

sed -e "s/\[spawner\-path\]/$spawner_path/g" \
    -e "s/\[client\-start\-dir\]/$client_start_dir/g" \
    -e "s/\[dmoindex\]/$dmoindex/g" \
    -e "s/\[dateFormat\]/$dateFormat/g" \
    -e "s/\[numberGroupSep\]/$numberGroupSep/g" \
    -e "s/\[numberDecimalSep\]/$numberDecimalSep/g" \
    -e "s/\[p2j\-entry\]/$p2j_entry/g" \
    -e "s/\[pkgroot\]/$pkgroot/g" \
    -e "s/\[propath\]/$propath/g" \
    -e "s/\[search\-path\]/$searchpath/g" \
    -e "s/\[path\-separator\]/$path_separator/g" \
    -e "s/\[file\-separator\]/$file_separator/g" \
    -e "s/\[case\-sensitive\]/$case_sensitive/g" \
    -e "s/\[dbname\]/$dbname/g" \
    -e "s/\[dbuser\]/$dbuser/g" \
    -e "s/\[dbuserpass\]/$dbuserpass/g" \
    -e "s/\[dbadmin\]/$dbadmin/g" \
    -e "s/\[dbadminpass\]/$dbadminpass/g" \
    -e "s/\[os\-user\]/$user/g" \
    deploy/server/directory.xml.template > deploy/server/directory.xml
   
export IFS=","
for dbname in $dbnames; do
   dbfile="deploy/server/directory_db.xml.$dbname"
   dirfile="deploy/server/directory.xml"
   
   sed -e "s/\[dbhost\]/$dbhost/g" \
       -e "s/\[dbport\]/$dbport/g" \
       -e "s/\[dbname\]/$dbname/g" \
       -e "s/\[dbuser\]/$dbuser/g" \
       -e "s/\[dbuserpass\]/$dbuserpass/g" \
       -e "s/\[dbadmin\]/$dbadmin/g" \
       -e "s/\[dbadminpass\]/$dbadminpass/g" \
       deploy/server/directory_db_$dbtype.xml.template > $dbfile
    
    java -Xmx256m \
         -classpath p2j/build/lib/p2j.jar \
         com.goldencode.p2j.directory.DirectoryCopy \
         copy $dbfile /server/standard/database/ $dirfile /server/standard/database/
    
    rm $dbfile
done
export IFS=

sed -e "s/\[propath\]/$propathcfg/g" \
    -e "s/\[include\-spec\]/$include_spec/g" \
    -e "s/\[pkgroot\]/$pkgroot/g" \
    -e "s/\[path\-separator\]/$path_separator/g" \
    -e "s/\[file\-separator\]/$file_separator/g" \
    -e "s/\[case\-sensitive\]/$case_sensitive/g" \
    -e "s/\[unix\-escapes\]/$unix_escapes/g" \
    cfg/p2j.cfg.xml.template > cfg/p2j.cfg.xml

dbnamespaces=""
export IFS=","
for dbname in $dbnames; do
   export IFS=
   dbnamespace=$(sed -e "s/\[dbname\]/$dbname/g" cfg/p2j.namespace.xml.template)
   dbnamespaces=$(echo $dbnamespaces && echo $dbnamespace)
   export IFS=","
done
export IFS=

sed_prepare "$dbnamespaces"
dbnamespaces="$sed_prepare_res"
dbnamespaces=$(echo $dbnamespaces | tr '\n' '\r')

cp cfg/p2j.cfg.xml cfg/p2j.cfg.xml.dbs
cat cfg/p2j.cfg.xml.dbs | sed -e "s/\[dbnamespaces\]/$dbnamespaces/g" | tr '\r' '\n' > cfg/p2j.cfg.xml
rm cfg/p2j.cfg.xml.dbs

echo "Class-Path: $appname.jar" > manifest/$appname.mf

if [ "$appname" != "hotel" ]
then
   rm hotel.bat hotel.ini hotel.input.linux hotel.input.windows hotel.pf prepare_hotel*.cmd \
      prepare_hotel*.sh data/hotel.df manifest/hotel.mf src/text-metrics.xml start_hotel_abl.cmd
   rm -fr data/dump/hotel
   rm -fr abldb/
   cd abl/ && rm -fr * && cd ..
   rm cfg/*.template deploy/server/*.template *.template \
      components-4gl-fwd.sh components-fwd-4gl.sh components-4gl-fwd.cmd components-fwd-4gl.cmd
fi

